@@ -2096,7 +2096,7 @@ gsqr(GEN x)
   if(tx==7)
   {
     y=cgetg(5,7);
-    y[2]=x[2];
+    y[2]=(long)pcopy((GEN)x[2]);
     if(!cmpis((GEN)x[2] ,2))
     {
       i=precp(x)+1;av=avma;
