@@ -130,8 +130,8 @@ extern  GEN     RAVYZARC;
 extern  long    NUMFUNC;
 extern  entree  fonctions[],**hashtable;
 extern  long    lontyp[],lontyp2[];
-
 extern  long    quitting_pari;
+
 extern  jmp_buf environnement;
 extern  FILE    *outfile, *logfile, *infile, *errfile;
 
@@ -176,7 +176,7 @@ extern const long VERYBIGINT;        /* 2^31-1               */
 extern  char    *helpmessage[]; /* un message pour chaque fonction predefinie */
 extern  char    *errmessage[];  /* un par numero d'erreur */
 extern  char    *pariversion;
-/*#define HAVEEXP2*/
+/* #define HAVEEXP2 */
 #ifdef HAVEEXP2
 #ifdef __cplusplus
 extern "C" {
@@ -189,8 +189,8 @@ extern "C" {
 inline double exp2(double x) {return exp(x*LOG2);}
 inline double log2(double x) {return log(x)/LOG2;}
 #else
-#define exp2(x) exp((double)(x)*LOG2)
-#define log2(x) log((double)(x))/LOG2
+#define exp2(x) (exp((double)(x)*LOG2))
+#define log2(x) (log((double)(x))/LOG2)
 #endif
 #endif
 
@@ -526,7 +526,10 @@ inline double log2(double x) {return log(x)/LOG2;}
 #define gcoeff(a,i,j)     (GEN)coeff(a,i,j)
 #define bern(i)           (GEN)(bernzone + (i)*(*(bernzone + 2)) + 3)
 
-#define copyifstack(x) (RAVYZARC=(GEN)(x),((RAVYZARC>=(GEN)bot)&&(RAVYZARC<(GEN)top))?lcopy(RAVYZARC):(long)RAVYZARC)
+/* #define copyifstack(x) (RAVYZARC=(GEN)(x),((RAVYZARC>=(GEN)bot)&&(RAVYZARC<(GEN)top))?lcopy(RAVYZARC):(long)RAVYZARC) */
+
+#define copyifstack(x) (lcopy((GEN)(x)))
+#define pcopy(x) (gcopy((GEN)(x)))
 
  /* The following definitions suppose well-behaved arrays and matrices */
  
@@ -633,8 +636,8 @@ void    lucas(long n, GEN *ln, GEN *ln1);
 GEN     base(GEN x, GEN *y),smallbase(GEN x, GEN *y),discf(GEN x),smalldiscf(GEN x),discf2(GEN x);
 GEN     hnf(GEN x),hnfhavas(GEN x),hnfnew(GEN x),hnfperm(GEN x);
 GEN     cleanmod(GEN x,long lim,GEN detmat,GEN detmatsur2);
-GEN     hnfmod(GEN x, GEN detmat),hnfmodid(GEN x,GEN p),smith(GEN x),smith2(GEN x);
-GEN     factoredbase(GEN x, GEN p, GEN *y),factoreddiscf(GEN x, GEN p),allbase(GEN x, long code, GEN *y),galois(GEN x, long prec),initalg(GEN x, long prec),initalgred(GEN x, long prec);
+GEN     hnfmod(GEN x, GEN detmat),hnfmodid(GEN x,GEN p),smith(GEN x),smith2(GEN x),gsmith(GEN x);
+GEN     factoredbase(GEN x, GEN p, GEN *y),factoreddiscf(GEN x, GEN p),allbase(GEN x, long code, GEN *y),galois(GEN x, long prec),initalg(GEN x, long prec),initalgred(GEN x, long prec),initalgredall(GEN x, long flun, long prec);
 GEN     tschirnhaus(GEN x),galoisconj(GEN x, long prec),galoisconj1(GEN x, long prec),galoisconj2(GEN x, long prec),galoisconjforce(GEN x, long prec),galoisapply(GEN nf, GEN aut, GEN x),initalgred2(GEN x, long prec);
 GEN     primedec(GEN nf,GEN p),idealmul(GEN nf,GEN ix,GEN iy),idealmulred(GEN nf, GEN ix, GEN iy, long prec), ideal_two_elt(GEN nf, GEN ix);
 GEN     idealmulh(GEN nf, GEN ix, GEN iy),element_mulh(GEN nf, long limi, long limj, GEN x, GEN y);
@@ -649,7 +652,7 @@ GEN     element_mul(GEN nf,GEN x,GEN y),element_sqr(GEN nf,GEN x),element_pow(GE
 GEN     rootsof1(GEN x),idealinv(GEN nf, GEN ix),oldidealinv(GEN nf, GEN ix);
 GEN     idealpow(GEN nf, GEN ix, GEN n),idealpowred(GEN nf, GEN ix, GEN n, long prec),idealpows(GEN nf, GEN ideal, long iexp);
 GEN     idealpowprime(GEN nf, GEN vp, GEN n,long prec),idealfactor(GEN nf, GEN x);
-GEN     idealhermite(GEN nf, GEN x),idealhermite2(GEN nf, GEN a, GEN b),idealadd(GEN nf, GEN x, GEN y), idealaddone(GEN nf, GEN x, GEN y), idealaddmultone(GEN nf, GEN list), idealdiv(GEN nf, GEN x, GEN y);
+GEN     idealhermite(GEN nf, GEN x),idealhermite2(GEN nf, GEN a, GEN b),idealadd(GEN nf, GEN x, GEN y), idealaddone(GEN nf, GEN x, GEN y), idealaddmultone(GEN nf, GEN list), idealdiv(GEN nf, GEN x, GEN y),  ideleaddone(GEN nf, GEN x, GEN idele);
 GEN     idealintersect(GEN nf, GEN x, GEN y), principalideal(GEN nf, GEN a);
 GEN     principalidele(GEN nf, GEN a),idealdivexact(GEN nf, GEN x, GEN y),idealnorm(GEN nf, GEN x);
 GEN     idealappr(GEN nf, GEN x),idealapprfact(GEN nf, GEN x), idealapprall(GEN nf, GEN x, long fl), idealchinese(GEN nf, GEN x, GEN y);
@@ -662,6 +665,8 @@ GEN     nfdetint(GEN nf,GEN pseudo);
 GEN     element_reduce(GEN nf, GEN x, GEN ideal);
 GEN     checknf(GEN nf), differente(GEN nf, GEN premiers);
 
+int     gpolcomp(GEN p1, GEN p2);
+
 long    idealval(GEN nf,GEN ix,GEN vp), isideal(GEN nf,GEN x);
 long    element_val(GEN nf, GEN x, GEN vp), element_val2(GEN nf, GEN x, GEN d, GEN vp);
 
@@ -669,9 +674,13 @@ long    rnfisfree(GEN bnf, GEN order);
 
 GEN     allbase4(GEN f, long code, GEN *y, GEN *ptw),base2(GEN x, GEN *y),rnfround2all(GEN nf, GEN pol, long all),rnfpseudobasis(GEN nf, GEN pol),rnfdiscf(GEN nf, GEN pol),rnfsimplifybasis(GEN bnf, GEN order),rnfsteinitz(GEN nf, GEN order),rnfbasis(GEN bnf, GEN order),rnfhermitebasis(GEN bnf, GEN order); 
 GEN     bsrch(GEN p, GEN fa, long Ka, GEN eta, long Ma),setup(GEN p,GEN f,GEN theta,GEN nut),eleval(GEN f,GEN h,GEN a),vstar(GEN p,GEN h),factcp(GEN p,GEN f,GEN beta),bestnu(GEN w),gcdpm(GEN f1,GEN f2,GEN pm);
-GEN     compositum(GEN pol1, GEN pol2);
+GEN     compositum(GEN pol1, GEN pol2),zidealstar(GEN nf, GEN x),zidealstarinit(GEN nf, GEN x),zidealstarinitold(GEN nf, GEN x),zideallog(GEN nf,GEN x,GEN bigideal),subcyclo(GEN p, GEN d);
 GEN     initzeta(GEN pol, long prec),gzetak(GEN nfz, GEN s, long prec),glambdak(GEN nfz, GEN s, long prec),gzetakall(GEN nfz, GEN s, long flag, long prec);
-GEN     nfreducemodpr(GEN nf, GEN x, GEN prhall),element_divmodpr(GEN nf, GEN x, GEN y, GEN prhall),element_powmodpr(GEN nf, GEN x, GEN k, GEN prhall);
+GEN     nfreducemodpr(GEN nf, GEN x, GEN prhall),element_divmodpr(GEN nf, GEN x, GEN y, GEN prhall),element_invmodpr(GEN nf, GEN y, GEN prhall),element_powmodpr(GEN nf, GEN x, GEN k, GEN prhall);
+GEN     reducemodmatrix(GEN x, GEN y),nfshanks(GEN nf,GEN x,GEN g0,GEN pr,GEN prhall);
+
+long nfsearch(GEN smalltable,GEN x);
+
 #define element_mulmodpr(nf,x,y,prhall) (nfreducemodpr(nf,element_mul(nf,x,y),prhall))
 #define element_sqrmodpr(nf,x,prhall) (nfreducemodpr(nf,element_sqr(nf,x),prhall))
 
@@ -685,9 +694,10 @@ GEN     lllgen(GEN x),lllkerimgen(GEN x),lllgramgen(GEN x),lllgramkerimgen(GEN x
 GEN     binome(GEN x, long k),gscal(GEN x, GEN y),cyclo(long n),vecsort(GEN x, GEN k);
 GEN     lindep(GEN x, long prec),lindep2(GEN x, long bit),lindep2bis(GEN x, long bit, long prec);
 GEN     algdep(GEN x, long n, long prec),algdep2(GEN x, long n, long bit),changevar(GEN x, GEN y),ordred(GEN x, long prec);
-GEN     polrecip(GEN x),reorder(GEN x),sort(GEN x),lexsort(GEN x),indexsort(GEN x),polsym(GEN x, long n);
+GEN     polrecip(GEN x),reorder(GEN x),sort(GEN x),lexsort(GEN x),indexsort(GEN x),indexlexsort(GEN x),polsym(GEN x, long n);
 GEN     minim(GEN a, long borne, long stockmax),minimprim(GEN a, long borne, long stockmax);
-GEN     polred(GEN x, long prec),factoredpolred(GEN x, GEN p, long prec),smallpolred(GEN x, long prec),polred2(GEN x, long prec),factoredpolred2(GEN x, GEN p, long prec), polredabs(GEN x, long prec);
+GEN     minim2(GEN a, long borne, long stockmax),minimprim(GEN a, long borne, long stockmax);
+GEN     polred(GEN x, long prec),factoredpolred(GEN x, GEN p, long prec),smallpolred(GEN x, long prec),polred2(GEN x, long prec),factoredpolred2(GEN x, GEN p, long prec),polredabs(GEN x, long prec),polredabs2(GEN x, long prec),polredabsfast(GEN x, long prec),polredabsall(GEN x, long prec);
 GEN     smallpolred2(GEN x, long prec),allpolred(GEN x, GEN *pta, long code, long prec),polymodrecip(GEN x),genrand(void),permute(long n, GEN x),permuteInv(GEN x);
 long    mymyrand();
 long    setprecr(long n),setserieslength(long n),ccontent(long* x,long n);
@@ -699,7 +709,9 @@ void    getheapaux(long* nombre, long* espace);
 GEN     somme(entree *ep, GEN x, GEN a, GEN b, char *ch),produit(entree *ep, GEN x, GEN a, GEN b, char *ch),suminf(entree *ep, GEN a, char *ch, long prec),prodinf(entree *ep, GEN a, char *ch, long prec),prodinf1(entree *ep, GEN a, char *ch, long prec),prodeuler(entree *ep, GEN a, GEN b, char *ch, long prec);
 GEN     vecteur(entree *ep, GEN nmax, char *ch),vvecteur(entree *ep, GEN nmax, char *ch),matrice(entree *ep1, entree *ep2, GEN nlig, GEN ncol, char *ch),divsomme(entree *ep, GEN num, char *ch);
 GEN     qromb(entree *ep, GEN a, GEN b, char *ch, long prec),qromo(entree *ep, GEN a, GEN b, char *ch, long prec),qromi(entree *ep, GEN a, GEN b, char *ch, long prec),rombint(entree *ep, GEN a, GEN b, char *ch, long prec);
-GEN     polint(GEN xa, GEN ya, GEN x, GEN *dy),plot(entree *ep, GEN a, GEN b, char *ch),ploth(entree *ep, GEN a, GEN b, char *ch, long prec),ploth2(entree *ep, GEN a, GEN b, char *ch, long prec),plothraw(GEN listx, GEN listy),zbrent(entree *ep, GEN a, GEN b, char *ch, long prec);
+GEN     polint(GEN xa, GEN ya, GEN x, GEN *dy),plot(entree *ep, GEN a, GEN b, char *ch);
+GEN     ploth(entree *ep, GEN a, GEN b, char *ch, long prec),plothmult(entree *ep, GEN a, GEN b, char *ch, long prec),ploth2(entree *ep, GEN a, GEN b, char *ch, long prec),plothraw(GEN listx, GEN listy);
+GEN     zbrent(entree *ep, GEN a, GEN b, char *ch, long prec);
 GEN     sumalt(entree *ep, GEN a, char *ch, long prec),sumalt1(entree *ep, GEN a, char *ch, long prec),sumalt2(entree *ep, GEN a, char *ch, long prec),sumalt3(entree *ep, GEN a, char *ch, long prec),sumpos(entree *ep, GEN a, char *ch, long prec),sumposold(entree *ep, GEN a, char *ch, long prec);
 GEN     forpari(entree *ep, GEN a, GEN b, char *ch),forstep(entree *ep, GEN a, GEN b, GEN s, char *ch),fordiv(entree *ep, GEN a, char *ch),forprime(entree *ep, GEN a, GEN b, char *ch),forvec(entree *ep, GEN x, char *ch);
 GEN     initrect(long ne, long x, long y),killrect(long ne),rectcursor(long ne),rectmove(long ne, GEN x, GEN y),rectrmove(long ne, GEN x, GEN y),rectpoint(long ne, GEN x, GEN y);
@@ -720,10 +732,12 @@ GEN     buchall(GEN P, GEN gcbach, GEN gcbach2, GEN gRELSUP, GEN gborne, long nb
 #define buchinit(P,gcbach,gcbach2,prec) buchall(P,gcbach,gcbach2,stoi(5),gzero,4,3,-1,prec)
 #define buchinitfu(P,gcbach,gcbach2,prec) buchall(P,gcbach,gcbach2,stoi(5),gzero,4,3,-2,prec)
 GEN     isprincipal(GEN bignf, GEN x),isprincipalgen(GEN bignf, GEN x);
-GEN     isunit(GEN bignf, GEN x), signunit(GEN bignf), buchnarrow(GEN bignf), buchfu(GEN bignf);
+GEN     isprincipalray(GEN bignf, GEN x),isprincipalraygen(GEN bignf, GEN x);
+GEN     isprincipalall(GEN bignf, GEN x,long flall),isprincipalrayall(GEN bignf, GEN x,long flall);
+GEN     isunit(GEN bignf, GEN x), signunit(GEN bignf), buchnarrow(GEN bignf), buchfu(GEN bignf),buchray(GEN bignf,GEN ideal),buchrayall(GEN bignf,GEN ideal,long flun),buchrayinit(GEN bignf,GEN ideal);
 int     compte(long **mat, long row, long longueur, long *firstnonzero);
 int     compte2(long **mat, long row, long longueur, long *firstnonzero);
-
+long    certifybuchall(GEN bnf);
 /* elliptic.c */
 
 GEN     ghell(GEN e, GEN a, long prec),ghell2(GEN e, GEN a, long prec),ghell3(GEN e, GEN a, long prec);
@@ -805,7 +819,7 @@ void    freeall(void), killall(void);
 
 void    killbloc(GEN x),newvalue(entree *ep, GEN val),killvalue(entree *ep);
 #ifdef __cplusplus
-extern "C" void    err(long numerr, ...);
+extern "C" volatile void err(long numerr, ...);
 #else
 extern
 #ifdef __GNUC__
